"""测试 MCP Server 工具."""

from unittest.mock import patch

import pytest

from modelaudit.models import (
    AuditResult,
    ComparisonResult,
    DetectionResult,
)

try:
    import mcp  # noqa: F401

    HAS_MCP = True
except ImportError:
    HAS_MCP = False

needs_mcp = pytest.mark.skipif(not HAS_MCP, reason="需要 mcp")


@needs_mcp
class TestCreateServer:
    def test_creates_server(self):
        from modelaudit.mcp_server import create_server

        server = create_server()
        assert server is not None

    def test_no_mcp_raises(self):
        from modelaudit import mcp_server

        original = mcp_server.HAS_MCP
        try:
            mcp_server.HAS_MCP = False
            with pytest.raises(ImportError, match="MCP 未安装"):
                mcp_server.create_server()
        finally:
            mcp_server.HAS_MCP = original


class TestDetectTextSource:
    def test_detect_tool_logic(self):
        """直接测试 detect 逻辑而不依赖 MCP 框架."""
        mock_results = [
            DetectionResult(
                text_id=1,
                text_preview="Hello world...",
                predicted_model="chatgpt",
                confidence=0.75,
                all_scores={"chatgpt": 0.75},
            ),
            DetectionResult(
                text_id=2,
                text_preview="Certainly! I'd...",
                predicted_model="chatgpt",
                confidence=0.80,
                all_scores={"chatgpt": 0.80},
            ),
        ]

        with patch("modelaudit.mcp_server.AuditEngine") as MockEngine:
            mock_engine = MockEngine.return_value
            mock_engine.detect.return_value = mock_results

            from modelaudit.mcp_server import AuditEngine

            engine = AuditEngine()
            results = engine.detect(["text1", "text2"])

            assert len(results) == 2
            assert results[0].predicted_model == "chatgpt"
            assert results[1].confidence == 0.80


class TestCompareModels:
    def test_compare_tool_logic(self):
        """测试 compare_models 工具的核心逻辑."""
        mock_result = ComparisonResult(
            model_a="gpt-4o",
            model_b="my-model",
            method="llmmap",
            similarity=0.85,
            is_derived=True,
            threshold=0.85,
            confidence=0.5,
            details={},
        )

        with patch("modelaudit.mcp_server.AuditEngine") as MockEngine:
            mock_engine = MockEngine.return_value
            mock_engine.compare.return_value = mock_result

            from modelaudit.mcp_server import AuditConfig, AuditEngine

            config = AuditConfig(provider="openai")
            engine = AuditEngine(config)
            result = engine.compare("gpt-4o", "my-model", method="llmmap", provider="openai")

            assert result.similarity == 0.85
            assert result.is_derived is True
            assert result.method == "llmmap"

    def test_compare_with_dli_method(self):
        """测试 DLI 方法参数传递."""
        mock_result = ComparisonResult(
            model_a="gpt-4o",
            model_b="my-model",
            method="dli",
            similarity=0.90,
            is_derived=True,
            threshold=0.80,
            confidence=0.5,
            details={},
        )

        with patch("modelaudit.mcp_server.AuditEngine") as MockEngine:
            mock_engine = MockEngine.return_value
            mock_engine.compare.return_value = mock_result

            from modelaudit.mcp_server import AuditConfig, AuditEngine

            config = AuditConfig(provider="openai")
            engine = AuditEngine(config)
            result = engine.compare("gpt-4o", "my-model", method="dli", provider="openai")

            assert result.method == "dli"
            mock_engine.compare.assert_called_once_with(
                "gpt-4o", "my-model", method="dli", provider="openai",
            )


class TestCompareModelsWhitebox:
    def test_whitebox_tool_logic(self):
        """测试 compare_models_whitebox 工具的核心逻辑."""
        mock_result = ComparisonResult(
            model_a="bert-base",
            model_b="distilbert",
            method="reef",
            similarity=0.92,
            is_derived=True,
            threshold=0.85,
            confidence=0.7,
            details={"layer_cka": {"layer_0": 0.95, "layer_1": 0.89}},
        )

        with patch("modelaudit.mcp_server.AuditEngine") as MockEngine:
            mock_engine = MockEngine.return_value
            mock_engine.compare.return_value = mock_result

            from modelaudit.mcp_server import AuditEngine

            engine = AuditEngine(use_cache=False)
            result = engine.compare("bert-base", "distilbert", method="reef", device="cpu")

            assert result.method == "reef"
            assert result.similarity == 0.92
            assert "layer_cka" in result.details


class TestAuditDistillation:
    def test_audit_tool_logic(self):
        """测试 audit_distillation 工具的核心逻辑."""
        mock_comparison = ComparisonResult(
            model_a="gpt-4o",
            model_b="my-model",
            method="llmmap",
            similarity=0.88,
            is_derived=True,
            threshold=0.85,
            confidence=0.6,
            details={},
        )
        mock_result = AuditResult(
            model_a="gpt-4o",
            model_b="my-model",
            comparisons=[mock_comparison],
            verdict="likely_derived",
            confidence=0.6,
            summary="审计对象: gpt-4o vs my-model\n判定结果: 可能存在蒸馏关系",
            details={},
        )

        with patch("modelaudit.mcp_server.AuditEngine") as MockEngine, \
             patch("modelaudit.mcp_server.generate_report") as mock_report:
            mock_engine = MockEngine.return_value
            mock_engine.audit.return_value = mock_result
            mock_report.return_value = "# 审计报告\n\n蒸馏关系: 是"

            from modelaudit.mcp_server import AuditConfig, AuditEngine, generate_report

            config = AuditConfig()
            engine = AuditEngine(config)
            result = engine.audit("gpt-4o", "my-model")

            assert result.verdict == "likely_derived"

            report = generate_report(result, "markdown")
            assert "审计报告" in report


class TestVerifyModel:
    def test_verify_tool_logic(self):
        """测试 verify_model 工具的核心逻辑."""
        mock_result = {
            "model": "gpt-4o",
            "verified": True,
            "claimed_family": "gpt-4",
            "best_match": "gpt-4",
            "claimed_score": 0.85,
            "best_score": 0.85,
            "all_scores": {"gpt-4": 0.85, "claude": 0.20, "llama": 0.15},
        }

        with patch("modelaudit.mcp_server.AuditEngine") as MockEngine:
            mock_engine = MockEngine.return_value
            mock_engine.verify.return_value = mock_result

            from modelaudit.mcp_server import AuditConfig, AuditEngine

            config = AuditConfig(provider="openai")
            engine = AuditEngine(config)
            result = engine.verify("gpt-4o", provider="openai")

            assert result["verified"] is True
            assert result["best_match"] == "gpt-4"
            assert "all_scores" in result


@needs_mcp
class TestToolList:
    def test_tool_count(self):
        """验证服务器创建成功 (5 个工具定义)."""
        from modelaudit.mcp_server import create_server

        server = create_server()
        assert server is not None



"""ModelAudit MCP Server — Model Context Protocol 服务."""

from typing import Any

try:
    from mcp.server import Server
    from mcp.server.stdio import stdio_server
    from mcp.types import TextContent, Tool

    HAS_MCP = True
except ImportError:
    HAS_MCP = False

from modelaudit.config import AuditConfig
from modelaudit.engine import AuditEngine
from modelaudit.report import generate_report


def create_server() -> "Server":
    """创建 MCP 服务器实例."""
    if not HAS_MCP:
        raise ImportError("MCP 未安装。请运行: pip install knowlyr-modelaudit[mcp]")

    server = Server("modelaudit")

    @server.list_tools()
    async def list_tools() -> list[Tool]:
        """列出可用的工具."""
        return [
            Tool(
                name="detect_text_source",
                description="检测文本数据来源 — 判断文本可能由哪个 LLM 生成",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "texts": {
                            "type": "array",
                            "items": {"type": "string"},
                            "description": "待检测的文本列表",
                        },
                    },
                    "required": ["texts"],
                },
            ),
            Tool(
                name="verify_model",
                description="验证模型身份 — 检查 API 背后是不是声称的模型",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "model": {
                            "type": "string",
                            "description": "模型名称 (如 gpt-4o, claude-3-opus)",
                        },
                        "provider": {
                            "type": "string",
                            "enum": ["openai", "anthropic", "custom"],
                            "description": "API 提供商 (默认: openai)",
                        },
                    },
                    "required": ["model"],
                },
            ),
            Tool(
                name="compare_models",
                description="比对两个模型的指纹相似度，判断是否存在蒸馏/派生关系",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "model_a": {"type": "string", "description": "模型 A 名称"},
                        "model_b": {"type": "string", "description": "模型 B 名称"},
                        "provider": {
                            "type": "string",
                            "description": "API 提供商 (默认: openai)",
                        },
                        "method": {
                            "type": "string",
                            "enum": ["llmmap", "dli", "style"],
                            "description": "指纹方法 (默认: llmmap)",
                        },
                    },
                    "required": ["model_a", "model_b"],
                },
            ),
            Tool(
                name="compare_models_whitebox",
                description="白盒比对两个本地模型 — 使用 REEF CKA 方法比较模型隐藏状态相似度（需要模型权重）",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "model_a": {
                            "type": "string",
                            "description": "本地模型路径或 HuggingFace 模型名 A",
                        },
                        "model_b": {
                            "type": "string",
                            "description": "本地模型路径或 HuggingFace 模型名 B",
                        },
                        "device": {
                            "type": "string",
                            "description": "计算设备 (默认: cpu)",
                        },
                    },
                    "required": ["model_a", "model_b"],
                },
            ),
            Tool(
                name="audit_distillation",
                description=(
                    "完整蒸馏审计 — 综合指纹比对 + 风格分析，生成详细审计报告。"
                    "支持跨 provider 审计（如 Kimi API + Claude API）。"
                ),
                inputSchema={
                    "type": "object",
                    "properties": {
                        "teacher": {
                            "type": "string",
                            "description": "教师模型 (疑似被蒸馏的源模型)",
                        },
                        "student": {
                            "type": "string",
                            "description": "学生模型 (疑似蒸馏产物)",
                        },
                        "teacher_provider": {
                            "type": "string",
                            "description": "教师模型 API 提供商 (默认: openai)",
                        },
                        "student_provider": {
                            "type": "string",
                            "description": "学生模型 API 提供商 (默认: openai)",
                        },
                        "teacher_api_base": {
                            "type": "string",
                            "description": "教师模型自定义 API 地址",
                        },
                        "student_api_base": {
                            "type": "string",
                            "description": "学生模型自定义 API 地址",
                        },
                        "format": {
                            "type": "string",
                            "enum": ["markdown", "json"],
                            "description": "报告格式 (默认: markdown)",
                        },
                    },
                    "required": ["teacher", "student"],
                },
            ),
            Tool(
                name="audit_memorization",
                description="检测模型是否记忆了训练数据 — 通过前缀补全和逐字检查评估记忆程度",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "text_samples": {
                            "type": "array",
                            "items": {"type": "string"},
                            "description": "候选训练文本列表",
                        },
                        "model": {
                            "type": "string",
                            "description": "待测模型名称 (默认: gpt-4o)",
                        },
                        "provider": {
                            "type": "string",
                            "enum": ["openai", "anthropic"],
                            "description": "API 提供商 (默认: openai)",
                        },
                        "method": {
                            "type": "string",
                            "enum": ["prefix_completion", "verbatim_check", "both"],
                            "description": "检测方法 (默认: prefix_completion)",
                        },
                    },
                    "required": ["text_samples"],
                },
            ),
            Tool(
                name="audit_report",
                description="生成完整的模型审计报告 — 汇总所有审计工具的结果",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "results": {
                            "type": "object",
                            "description": "各审计工具的结果字典 (tool_name -> result_text)",
                        },
                        "model_name": {
                            "type": "string",
                            "description": "被审计模型名称",
                        },
                        "audit_date": {
                            "type": "string",
                            "description": "审计日期 (默认: 今天)",
                        },
                    },
                    "required": ["results", "model_name"],
                },
            ),
            Tool(
                name="audit_watermark",
                description="检测文本中是否包含 AI 水印（统计特征和模式匹配）",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "texts": {
                            "type": "array",
                            "items": {"type": "string"},
                            "description": "待检测的文本列表",
                        },
                        "method": {
                            "type": "string",
                            "enum": ["statistical", "pattern", "both"],
                            "description": "检测方法（默认 both）",
                            "default": "both",
                        },
                    },
                    "required": ["texts"],
                },
            ),
        ]

    @server.call_tool()
    async def call_tool(name: str, arguments: dict[str, Any]) -> list[TextContent]:
        """调用工具."""

        if name == "detect_text_source":
            texts = arguments["texts"]
            engine = AuditEngine()
            results = engine.detect(texts)

            lines = ["## 文本来源检测结果", ""]
            lines.append("| # | 预测模型 | 置信度 | 预览 |")
            lines.append("|---|---------|--------|------|")

            for r in results:
                lines.append(
                    f"| {r.text_id} | {r.predicted_model} | {r.confidence:.2%} | {r.text_preview} |"
                )

            # 统计
            model_counts: dict[str, int] = {}
            for r in results:
                model_counts[r.predicted_model] = model_counts.get(r.predicted_model, 0) + 1

            lines.extend(["", "### 来源分布"])
            for model, count in sorted(model_counts.items(), key=lambda x: -x[1]):
                pct = count / len(results) * 100
                lines.append(f"- {model}: {count} ({pct:.1f}%)")

            return [TextContent(type="text", text="\n".join(lines))]

        elif name == "verify_model":
            model = arguments["model"]
            provider = arguments.get("provider", "openai")

            config = AuditConfig(provider=provider)
            engine = AuditEngine(config)
            result = engine.verify(model, provider=provider)

            verified = result["verified"]
            icon = "✓" if verified else "✗"

            text = f"""## 模型身份验证

{icon} **{'验证通过' if verified else '验证未通过'}**

- 声称模型: {model}
- 最佳匹配: {result['best_match']} (分数: {result['best_score']})

### 风格匹配分数
"""
            for name_k, score in result.get("all_scores", {}).items():
                text += f"- {name_k}: {score:.4f}\n"

            return [TextContent(type="text", text=text)]

        elif name == "compare_models":
            model_a = arguments["model_a"]
            model_b = arguments["model_b"]
            provider = arguments.get("provider", "openai")
            method = arguments.get("method", "llmmap")

            config = AuditConfig(provider=provider)
            engine = AuditEngine(config)
            result = engine.compare(model_a, model_b, method=method, provider=provider)

            derived_text = "可能存在派生关系" if result.is_derived else "可能是独立模型"
            icon = "⚠️" if result.is_derived else "✓"

            text = f"""## 模型比对结果

{icon} **{derived_text}**

- 模型 A: {model_a}
- 模型 B: {model_b}
- 相似度: {result.similarity:.4f}
- 阈值: {result.threshold}
- 置信度: {result.confidence:.4f}
"""
            return [TextContent(type="text", text=text)]

        elif name == "compare_models_whitebox":
            model_a = arguments["model_a"]
            model_b = arguments["model_b"]
            device = arguments.get("device", "cpu")

            engine = AuditEngine(use_cache=False)
            result = engine.compare(
                model_a, model_b, method="reef", device=device,
            )

            derived_text = "可能存在派生关系" if result.is_derived else "可能是独立模型"
            icon = "⚠️" if result.is_derived else "✓"

            text = f"""## 白盒模型比对结果 (REEF CKA)

{icon} **{derived_text}**

- 模型 A: {model_a}
- 模型 B: {model_b}
- 相似度: {result.similarity:.4f}
- 阈值: {result.threshold}
- 置信度: {result.confidence:.4f}
"""
            if "layer_cka" in result.details:
                text += "\n### 逐层 CKA 相似度\n"
                for layer, cka in result.details["layer_cka"].items():
                    text += f"- {layer}: {cka:.4f}\n"

            return [TextContent(type="text", text=text)]

        elif name == "audit_distillation":
            teacher = arguments["teacher"]
            student = arguments["student"]
            output_format = arguments.get("format", "markdown")

            config = AuditConfig()
            engine = AuditEngine(config)
            result = engine.audit(
                teacher, student,
                teacher_provider=arguments.get("teacher_provider"),
                teacher_api_key=arguments.get("teacher_api_key"),
                teacher_api_base=arguments.get("teacher_api_base"),
                student_provider=arguments.get("student_provider"),
                student_api_key=arguments.get("student_api_key"),
                student_api_base=arguments.get("student_api_base"),
            )

            report = generate_report(result, output_format)
            return [TextContent(type="text", text=report)]

        elif name == "audit_memorization":
            from difflib import SequenceMatcher
            text_samples = arguments["text_samples"]
            model = arguments.get("model", "gpt-4o")
            provider = arguments.get("provider", "openai")
            method = arguments.get("method", "prefix_completion")

            # Try to call model API
            try:
                from modelaudit.methods.llmmap import _call_model_api_once
                has_client = True
            except ImportError:
                has_client = False

            if not has_client:
                lines = ["## 记忆检测 — 交互模式", "", "未找到 LLM 客户端库，以下是供手动执行的提示：", ""]
                for i, sample in enumerate(text_samples):
                    mid = len(sample) // 2
                    lines.append(f"### 样本 {i + 1}")
                    lines.append(f"```\n请续写以下文本：\n\n{sample[:mid]}\n```\n")
                return [TextContent(type="text", text="\n".join(lines))]

            results = []
            for i, sample in enumerate(text_samples):
                mid = len(sample) // 2
                prefix = sample[:mid]
                suffix = sample[mid:]
                if method in ("prefix_completion", "both"):
                    prompt = f"请续写以下文本，直接输出续写部分：\n\n{prefix}"
                    completion = _call_model_api_once(model, prompt, provider=provider)
                    score = SequenceMatcher(None, suffix.strip(), completion.strip()).ratio()
                    results.append({"index": i + 1, "method": "prefix_completion", "score": round(score, 4)})
                if method in ("verbatim_check", "both"):
                    prompt = f"请用你自己的话复述以下文本的关键信息：\n\n{sample[:200]}"
                    response = _call_model_api_once(model, prompt, provider=provider)
                    words = sample.split()
                    phrase_len = min(8, len(words) // 4) or 3
                    matches = total = 0
                    for j in range(0, len(words) - phrase_len + 1, phrase_len):
                        phrase = " ".join(words[j:j + phrase_len])
                        total += 1
                        if phrase in response:
                            matches += 1
                    score = matches / total if total else 0
                    results.append({"index": i + 1, "method": "verbatim_check", "score": round(score, 4)})

            lines = [f"## 训练数据记忆检测结果 ({model})", "", f"- 检测方法: {method}", f"- 样本数量: {len(text_samples)}", "",
                     "| # | 方法 | 记忆分数 | 评级 |", "|---|------|---------|------|"]
            for r in results:
                level = "high" if r["score"] >= 0.7 else ("medium" if r["score"] >= 0.4 else "low")
                icon = {"high": "🔴", "medium": "🟡", "low": "🟢"}[level]
                lines.append(f"| {r['index']} | {r['method']} | {r['score']:.4f} | {icon} {level} |")
            if results:
                avg = sum(r["score"] for r in results) / len(results)
                lines.extend(["", f"- 平均记忆分数: {avg:.4f}"])
            return [TextContent(type="text", text="\n".join(lines))]

        elif name == "audit_report":
            from datetime import date
            results = arguments["results"]
            model_name = arguments["model_name"]
            audit_date = arguments.get("audit_date", date.today().isoformat())
            sections = {
                "detect_text_source": "文本来源检测",
                "verify_model": "模型身份验证",
                "audit_distillation": "蒸馏分析",
                "compare_models": "模型指纹比对",
                "audit_memorization": "记忆检测",
            }
            lines = [f"# 模型审计报告：{model_name}", "", f"**审计日期**: {audit_date}", ""]
            for key, title in sections.items():
                lines.append(f"## {title}")
                lines.append("")
                if key in results:
                    lines.append(results[key])
                else:
                    lines.append("*未执行此项检查。*")
                lines.append("")
            return [TextContent(type="text", text="\n".join(lines))]

        elif name == "audit_watermark":
            texts = arguments["texts"]
            method = arguments.get("method", "both")

            results = []
            for i, text in enumerate(texts):
                score = 0.0
                signals = []

                if method in ("statistical", "both"):
                    # Statistical: check token distribution uniformity
                    words = text.split()
                    if len(words) > 20:
                        # Check for unusually uniform word length distribution
                        lengths = [len(w) for w in words]
                        mean_len = sum(lengths) / len(lengths)
                        variance = sum((l - mean_len) ** 2 for l in lengths) / len(lengths)
                        # Low variance in word lengths can indicate watermarking
                        if variance < 2.0:
                            score += 0.3
                            signals.append(f"低词长方差 ({variance:.2f})")

                        # Check for repetitive n-gram patterns
                        bigrams = [f"{words[j]} {words[j+1]}" for j in range(len(words)-1)]
                        unique_ratio = len(set(bigrams)) / len(bigrams) if bigrams else 1
                        if unique_ratio < 0.5:
                            score += 0.2
                            signals.append(f"低 bigram 唯一率 ({unique_ratio:.2f})")

                if method in ("pattern", "both"):
                    # Pattern: check for known watermark patterns
                    # Unicode zero-width characters
                    zwc_count = sum(1 for c in text if c in "\u200b\u200c\u200d\ufeff")
                    if zwc_count > 0:
                        score += 0.5
                        signals.append(f"零宽字符: {zwc_count} 个")

                    # Unusual whitespace patterns
                    double_space = text.count("  ")
                    if double_space > 3:
                        score += 0.2
                        signals.append(f"双空格: {double_space} 处")

                level = "high" if score >= 0.5 else ("medium" if score >= 0.3 else "low")
                results.append({"index": i + 1, "score": round(score, 2), "level": level, "signals": signals})

            lines = [
                "## AI 水印检测结果", "",
                f"- 检测方法: {method}", f"- 文本数: {len(texts)}", "",
                "| # | 水印分数 | 级别 | 信号 |",
                "|---|---------|------|------|",
            ]
            for r in results:
                icon = {"high": "🔴", "medium": "🟡", "low": "🟢"}[r["level"]]
                sigs = "; ".join(r["signals"]) if r["signals"] else "无"
                lines.append(f"| {r['index']} | {r['score']:.2f} | {icon} {r['level']} | {sigs} |")

            return [TextContent(type="text", text="\n".join(lines))]

        else:
            return [TextContent(type="text", text=f"未知工具: {name}")]

    return server


async def serve():
    """启动 MCP 服务器."""
    if not HAS_MCP:
        raise ImportError("MCP 未安装。请运行: pip install knowlyr-modelaudit[mcp]")

    server = create_server()
    async with stdio_server() as (read_stream, write_stream):
        await server.run(read_stream, write_stream)


def main():
    """主入口."""
    import asyncio

    asyncio.run(serve())


if __name__ == "__main__":
    main()



"""风格分析方法 — 基于文本特征检测模型来源.

参考 "Who Taught You That?" (ACL 2025 Findings) 的 PoS 模板思路:
不同 LLM 生成的文本具有独特的风格签名，包括:
- 词频分布
- 句法结构偏好
- 特征性短语和过渡词
- 格式化习惯（列表、Markdown 等）
"""

import re
from typing import Any

from modelaudit.models import DetectionResult

# 已知 LLM 风格特征库
# markers: 独特短语 (越独特越好，避免跨模型重叠)
# refusal_patterns: 拒绝时的典型表达
# structural: 格式偏好
# lang: 主要语言 ("en", "zh", "both")
MODEL_STYLE_SIGNATURES: dict[str, dict[str, Any]] = {
    "gpt-4": {
        "markers": [
            "certainly! here's", "comprehensive breakdown",
            "it's important to note", "it's worth noting",
            "let me know if you'd like", "keep in mind",
            "in more detail", "would you like me to",
            "let me walk you through", "active area of research",
        ],
        "refusal_patterns": [
            "i can't assist", "i'm not able to", "as an ai language model",
        ],
        "structural": {
            "tends_markdown": True,
            "tends_numbered_lists": True,
            "tends_code_blocks": True,
            "verbose": True,
        },
        "lang": "en",
    },
    "gpt-3.5": {
        "markers": [
            "certainly!", "sure!", "of course!", "absolutely!",
            "no problem!", "hope that helps!",
        ],
        "refusal_patterns": [
            "as an ai language model", "i don't have the ability",
        ],
        "structural": {
            "tends_markdown": False,
            "tends_numbered_lists": True,
            "tends_code_blocks": False,
            "verbose": False,
        },
        "lang": "en",
    },
    "claude": {
        "markers": [
            "i'd be happy to help", "let me think through this",
            "i should note", "i want to be straightforward",
            "nuanced", "i want to be careful",
            "different perspectives", "ethical implications",
            "would you like me to continue",
            "take it in a different direction",
        ],
        "refusal_patterns": [
            "i don't think i should", "i'd rather not",
            "i want to be helpful but",
        ],
        "structural": {
            "tends_markdown": True,
            "tends_numbered_lists": False,
            "tends_code_blocks": True,
            "verbose": True,
        },
        "lang": "en",
    },
    "llama": {
        "markers": [
            "sure thing!", "no problem", "pretty cool",
            "check out this", "here you go",
            "let me know if you need anything else",
            "so basically", "think of it like",
        ],
        "refusal_patterns": [
            "i cannot", "i'm just an ai", "it's not appropriate",
        ],
        "structural": {
            "tends_markdown": False,
            "tends_numbered_lists": False,
            "tends_code_blocks": True,
            "verbose": False,
        },
        "lang": "en",
    },
    "gemini": {
        "markers": [
            "great question!", "here's what you need to know",
            "**key applications**", "key applications",
            "it's worth noting that", "progress is accelerating",
            "i can provide a code implementation",
            "systematically", "noisy and error-prone",
        ],
        "refusal_patterns": [
            "i'm a large language model", "i'm designed to be helpful",
        ],
        "structural": {
            "tends_markdown": True,
            "tends_numbered_lists": True,
            "tends_code_blocks": False,
            "verbose": True,
        },
        "lang": "en",
    },
    "qwen": {
        "markers": [
            "好的，我来", "为您详细", "具体来说",
            "以下是一个高效的", "这个实现", "时间复杂度",
            "核心思想", "优势在于",
        ],
        "refusal_patterns": [
            "作为ai助手", "我无法提供",
        ],
        "structural": {
            "tends_markdown": True,
            "tends_numbered_lists": True,
            "tends_code_blocks": True,
            "verbose": False,
        },
        "lang": "zh",
    },
    "deepseek": {
        "markers": [
            "嗯，让我仔细想想", "从多个角度", "本质上是",
            "状态转移方程", "边界条件", "如果需要优化",
            "从技术层面看", "从实际应用角度",
        ],
        "refusal_patterns": [
            "作为ai助手", "我无法提供",
        ],
        "structural": {
            "tends_markdown": True,
            "tends_numbered_lists": True,
            "tends_code_blocks": True,
            "verbose": True,
        },
        "lang": "zh",
    },
    "mistral": {
        "markers": [
            "to answer your question",
            "in short", "the answer is", "straightforward",
        ],
        "refusal_patterns": [
            "i cannot", "i must decline", "it would be inappropriate",
        ],
        "structural": {
            "tends_markdown": False,
            "tends_numbered_lists": False,
            "tends_code_blocks": False,
            "verbose": False,
        },
        "lang": "en",
    },
    "yi": {
        "markers": [
            "to put it simply", "in a nutshell",
            "i'd like to point out",
        ],
        "refusal_patterns": [
            "as an ai", "i'm not able to", "i cannot assist with",
        ],
        "structural": {
            "tends_markdown": True,
            "tends_numbered_lists": True,
            "tends_code_blocks": False,
            "verbose": True,
        },
        "lang": "en",
    },
    "phi": {
        "markers": [
            "in conclusion", "the answer is simply",
        ],
        "refusal_patterns": [
            "i cannot", "i'm unable to", "as a language model",
        ],
        "structural": {
            "tends_markdown": False,
            "tends_numbered_lists": False,
            "tends_code_blocks": False,
            "verbose": False,
        },
        "lang": "en",
    },
    "cohere": {
        "markers": [
            "here's what i found", "to elaborate",
            "happy to help with that",
        ],
        "refusal_patterns": [
            "i'm not able to", "i'd prefer not to", "i cannot help with",
        ],
        "structural": {
            "tends_markdown": True,
            "tends_numbered_lists": True,
            "tends_code_blocks": False,
            "verbose": True,
        },
        "lang": "en",
    },
    "chatglm": {
        "markers": [
            "好的", "以下是", "总结一下",
            "首先我们需要", "希望对您有帮助",
        ],
        "refusal_patterns": [
            "作为ai助手", "我无法",
        ],
        "structural": {
            "tends_markdown": True,
            "tends_numbered_lists": True,
            "tends_code_blocks": False,
            "verbose": True,
        },
        "lang": "zh",
    },
}


def _detect_lang(text: str) -> str:
    """检测文本主要语言: 'zh' 或 'en'."""
    cjk_count = sum(1 for ch in text if "\u4e00" <= ch <= "\u9fff")
    # 绝对数量兜底: 即使代码多, 10 个汉字也算中文
    if cjk_count >= 10:
        return "zh"
    total = len(text) or 1
    return "zh" if cjk_count / total > 0.15 else "en"


def _compute_style_scores(text: str) -> dict[str, float]:
    """计算文本与各模型风格的匹配分数."""
    text_lower = text.lower()
    words = text_lower.split()
    total_words = len(words) or 1
    text_lang = _detect_lang(text)

    # 预计算结构特征 (只算一次)
    has_md = bool(re.search(r"^#+\s", text, re.MULTILINE))
    has_numbered = bool(re.search(r"^\s*\d+[.)]\s", text, re.MULTILINE))
    has_code_blocks = "```" in text
    is_verbose = total_words > 150

    # 检测是否有拒绝内容 (决定是否启用拒绝分数)
    has_refusal_hint = any(
        kw in text_lower
        for kw in ("i cannot", "i can't", "unable to", "我无法", "作为ai")
    )

    scores: dict[str, float] = {}

    for model_name, sig in MODEL_STYLE_SIGNATURES.items():
        score = 0.0
        model_lang = sig.get("lang", "en")

        # ── 1. 语言匹配 (权重 0.20) ──
        if text_lang == model_lang:
            score += 0.20
        elif model_lang == "both":
            score += 0.10
        # 语言不匹配: +0, 形成天然屏障

        # ── 2. 标记词匹配 (权重 0.50) ──
        # 用固定分母 (3) 归一化, 避免 marker 多的模型吃亏
        marker_hits = sum(1 for m in sig["markers"] if m in text_lower)
        score += min(marker_hits / 3, 1.0) * 0.50

        # ── 3. 结构特征匹配 (权重 0.20) ──
        # 只对文本实际展现的特征计分, 避免 "双否" 虚假匹配
        structural = sig["structural"]
        struct_score = 0.0
        for text_has, key in [
            (has_md, "tends_markdown"),
            (has_numbered, "tends_numbered_lists"),
            (has_code_blocks, "tends_code_blocks"),
            (is_verbose, "verbose"),
        ]:
            model_tends = structural.get(key, False)
            if text_has and model_tends:
                struct_score += 0.05   # 正向匹配
            elif text_has and not model_tends:
                struct_score -= 0.02   # 文本有但模型不倾向 → 轻微惩罚
        score += struct_score

        # ── 4. 拒绝模式 (权重 0.10, 仅文本含拒绝时生效) ──
        if has_refusal_hint:
            refusal_hits = sum(1 for p in sig["refusal_patterns"] if p in text_lower)
            score += refusal_hits / max(len(sig["refusal_patterns"]), 1) * 0.10

        scores[model_name] = round(score, 4)

    return scores


def detect_text_source(texts: list[str]) -> list[DetectionResult]:
    """检测文本来源 — 判断文本可能由哪个模型生成.

    Args:
        texts: 待检测的文本列表

    Returns:
        每条文本的检测结果
    """
    results: list[DetectionResult] = []

    for i, text in enumerate(texts):
        scores = _compute_style_scores(text)

        if scores:
            best_model = max(scores, key=lambda k: scores[k])
            best_score = scores[best_model]
        else:
            best_model = "unknown"
            best_score = 0.0

        preview = text[:80] + "..." if len(text) > 80 else text
        preview = preview.replace("\n", " ")

        results.append(
            DetectionResult(
                text_id=i,
                text_preview=preview,
                predicted_model=best_model,
                confidence=best_score,
                scores=scores,
            )
        )

    return results


def compute_style_fingerprint(texts: list[str]) -> dict[str, float]:
    """从一组文本中提取风格指纹向量."""
    all_scores: dict[str, list[float]] = {}

    for text in texts:
        scores = _compute_style_scores(text)
        for model, score in scores.items():
            all_scores.setdefault(model, []).append(score)

    # 聚合为平均分
    return {
        model: round(sum(s) / len(s), 4) for model, s in all_scores.items() if s
    }



"""测试审计引擎."""

from unittest.mock import patch

import pytest

from modelaudit.config import AuditConfig
from modelaudit.engine import AuditEngine
from modelaudit.models import Fingerprint


class TestAuditEngine:
    def test_init_default(self):
        engine = AuditEngine()
        assert engine.config.blackbox_method == "llmmap"

    def test_init_custom_config(self):
        config = AuditConfig(provider="anthropic", similarity_threshold=0.9)
        engine = AuditEngine(config)
        assert engine.config.provider == "anthropic"

    def test_detect(self):
        engine = AuditEngine()
        texts = [
            "Certainly! I'd be happy to help with that question.",
            "I think that's a very interesting perspective to consider.",
        ]
        results = engine.detect(texts)
        assert len(results) == 2
        assert all(r.predicted_model for r in results)

    def test_detect_empty(self):
        engine = AuditEngine()
        results = engine.detect([])
        assert results == []

    def test_fingerprint_empty_model_name(self):
        engine = AuditEngine()
        with pytest.raises(ValueError, match="模型名称不能为空"):
            engine.fingerprint("")

    def test_fingerprint_whitespace_model_name(self):
        engine = AuditEngine()
        with pytest.raises(ValueError, match="模型名称不能为空"):
            engine.fingerprint("   ")


class TestAuditConfig:
    def test_default_values(self):
        config = AuditConfig()
        assert config.provider == "openai"
        assert config.similarity_threshold == 0.85
        assert config.output_format == "markdown"
        assert config.cache_ttl == 0

    def test_custom_values(self):
        config = AuditConfig(
            provider="anthropic",
            similarity_threshold=0.9,
            num_probes=4,
        )
        assert config.provider == "anthropic"
        assert config.num_probes == 4

    def test_api_timeout_and_retries(self):
        config = AuditConfig(api_timeout=120, api_max_retries=5)
        assert config.api_timeout == 120
        assert config.api_max_retries == 5

    def test_api_timeout_default(self):
        config = AuditConfig()
        assert config.api_timeout == 60
        assert config.api_max_retries == 3

    def test_cache_ttl(self):
        config = AuditConfig(cache_ttl=3600)
        assert config.cache_ttl == 3600

    def test_cache_ttl_passed_to_engine(self):
        config = AuditConfig(cache_ttl=7200)
        engine = AuditEngine(config, use_cache=True)
        assert engine.cache is not None
        assert engine.cache.ttl == 7200


class TestAuditDLIIntegration:
    """测试 audit() 多方法集成."""

    def test_audit_includes_dli_comparison(self):
        """audit 结果应包含 LLMmap 和 DLI 两个比对."""
        # Mock fingerprint 返回带 raw_responses 的数据
        mock_fp = Fingerprint(
            model_id="test",
            method="llmmap",
            fingerprint_type="blackbox",
            data={
                "vector": {
                    "avg_length_chars": 100.0,
                    "avg_length_words": 20.0,
                    "avg_length_sentences": 3.0,
                    "avg_avg_word_length": 5.0,
                    "avg_avg_sentence_length": 15.0,
                    "avg_unique_word_ratio": 0.7,
                    "avg_punctuation_ratio": 0.03,
                    "avg_newline_ratio": 0.01,
                },
                "raw_responses": [
                    "Certainly! Here's the answer.",
                    "I'd be happy to help with that.",
                    "Let me explain this concept.",
                ],
                "probe_ids": ["p1", "p2", "p3"],
            },
        )

        with patch.object(AuditEngine, "fingerprint", return_value=mock_fp):
            engine = AuditEngine(use_cache=False)
            result = engine.audit("model-a", "model-b")

            methods = [c.method for c in result.comparisons]
            assert "llmmap" in methods
            assert "dli" in methods
            assert len(result.comparisons) >= 2

    def test_audit_skips_dli_when_no_responses(self):
        """无 raw_responses 时 DLI 应跳过并记录原因."""
        mock_fp = Fingerprint(
            model_id="test",
            method="llmmap",
            fingerprint_type="blackbox",
            data={"vector": {}, "raw_responses": [], "probe_ids": []},
        )

        with patch.object(AuditEngine, "fingerprint", return_value=mock_fp):
            engine = AuditEngine(use_cache=False)
            result = engine.audit("model-a", "model-b")

            methods = [c.method for c in result.comparisons]
            assert "dli" not in methods
            assert "skipped_methods" in result.details
            assert any("DLI" in s for s in result.details["skipped_methods"])
